# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.protocol.base import BaseProtocol
from driverng.utils import ACK, ETX, STX, byte2log


class AlifaxProtocol(BaseProtocol):
    def send_trames(self, trame):
        self.write(trame)
        return True

    def recv_trames(self):
        data = self.read()
        if not data or data[0] != STX:
            self.logger.warning("<stx> expected in %s", byte2log(data))
            return False

        data += self.readline(break_func=lambda s: s and s[-1] == ETX)
        # acquitement autorise uniquement pour le message de resultat R
        mesgs = []
        if data[1] == b"R":
            self.write(ACK)
        # il peut y avoir plusieurs messages à la fois
        for d in data.split(b"\x02"):
            if d:
                self.logger.info("alifaxrecv: %s", byte2log(d))
                mesgs.append(self.get_payload(d))
        return mesgs

    def get_payload(self, data):
        # <stx> <payload><checksum>   <etx>
        fin = data.find(ETX)
        if fin >= 0:
            return data[:fin]
        return data

    def build_trames(self, message):
        # <stx><payload><checksum><etx>
        return b"%c%s%c%c" % (STX, message, self.checksum_xor(message), ETX)

    @staticmethod
    def checksum_xor(data):
        # Exclusive XOR of all characters of the message
        # excluded <stx> and <etx> and <checksum>. If the checksum
        # calculated is 0x03 (ETX), or 0x02 (STX) or 0x00 (NULL), the
        # value will be automatically replaced with 0x7F hex value
        chk = 0
        for c in bytearray(data):
            chk ^= c
        if chk in (0x02, 0x03, 0x00):
            chk = 0x7F
        return chk
